% -*- coding: utf-8 -*-
%
% Calculation of mean and autocorrelation function for a rando process
% associated with the random experiment of rolling a die
% with 6 signals, one for each possible outcome of the random experiment
%    
% Cálculo de media y de función de autocorrelación para un proceso
% aleatorio asociado al experimento aleatorio del lanzamiento de un dado
% con 6 señales, una para cada posible resultado del experimento aleatorio
%
%--------------------------------------------------------------------------
%
% LABORATORIO DE TEORÍA DE LA COMUNICACIÓN
%
%        Versión: 1.0
%  Realizado por: Marcelino Lázaro
%                 Departamento de Teoría de la Señal y Comunicaciones
%                 Universidad Carlos III de Madrid
%      Creación : febrero 2023
% Actualización : febrero 2023
%
%--------------------------------------------------------------------------

Ninstantes = 128;

Nfft = 128;

w = 2*pi/32;

T = 1;
t = 1:T:Ninstantes;

fs = 1/T;
fnorm=-1:2/Nfft:(1-1/Nfft);
f=fs/2*fnorm;

x = zeros(6,Ninstantes);
X = zeros(6,Nfft);
% Bucle para generar cada una de las 6 señales
% Loop to generate each of the 6 signals
for k = 1:6
    fase = 2*pi/6*(k-1);
    x(k,:) = 0.5 + sin(w*t - fase);
    X(k,:) = fftshift(fft(x(k,:),Nfft))/(Nfft);
    figure(1),subplot(6,1,k)
    plot(t,x(k,:)), axis([0,Ninstantes,-1,2])
    ylabel(['x_',num2str(k),'(t)'])
    figure(2),subplot(6,1,k)
    plot(f,abs(X(k,:)))
    ylabel(['|X_',num2str(k),'(j\omega)|'])
end
figure(1),xlabel('t (s)')
subplot(6,1,1),title('Signals in the random process')

figure(2),xlabel('f (Hz)')
subplot(6,1,1),title('Modulus of the FT of each signal')

% Cálculo de la media de las 6 señales
xM = mean(x,1);

% Representación de la media de las señales
figure(3),subplot(1,1,1),plot(t,xM), axis([0,Ninstantes,-1,2])
xlabel('t (s)')
title('Mean of the process: m_X(t)')

% Estima de la función de autocorrelación de las 6 señales
% Estimate of the autocorrelation function of the 6 signals      
Rx = zeros(Ninstantes,Ninstantes);

for t1 = 1:Ninstantes
    for t2 = 1:Ninstantes
        Rx(t1,t2) = mean(x(:,t1).*x(:,t2));
    end
end

% Representación de la estima de la función de autocorrelación
% Representation of the estimate of the autocorrelation function    
figure(4),mesh(t,t,Rx)
xlabel('t_1 (s)')
ylabel('t_2 (s)')
title('Autocorrelation function: R_X(t_1,t_2)')

% Estima de la autocorrelación en función de tau para t = 1
% Estimate of autocorrelation as a function of tau for t = 1      
tref = 1;

Rxref = zeros(1,51);
tau=0:100;

for k = 1:length(tau)
    Rxref(1,k) = mean(x(:,tref+tau(k)).*x(:,tref));
end

% Representación de la estima de la autocorrelación para t = 1 en función
% de tau
% Representation of the autocorrelation estimate for t = 1 as a function
% tau  
figure(5), plot(tau, Rxref)
xlabel('\tau (s)')
title(['R_X(t+\tau,t) for t=',num2str(tref),' s'])

