%--------------------------------------------------------------------------
% Demonstration of Analog Modulations concepts for
% COMMUNICATION THEORY laboratory
%
% Demostración de conceptos de Modulaciones Analógicas para 
% el laboratorio de TEORÍA DE LA COMUNICACIÓN
%
% Modulación de fase (FM)
%
%--------------------------------------------------------------------------
%
% LABORATORIO DE TEORÍA DE LA COMUNICACIÓN
%
%        Versión: 1.0
%  Realizado por: Marcelino Lázaro
%                 Departamento de Teoría de la Señal y Comunicaciones
%                 Universidad Carlos III de Madrid
%      Creación : marzo 2023
% Actualización : marzo 2023
%
%--------------------------------------------------------------------------

% Parámetros temporales
iIni=0;         % Instance inicial
tFin=10;        % Instante final

fc=100;         % Frecuencia de portadora 
faseTx=0;       % Fase de la portadora en transmisión
fm=5;           % Frecuencia de la señal moduladora sinusoidal
 
desvFreq = 20;  % Desviación de frecuencia (en Hz)

fs=2000;        % Frecuencia de muestreo 
Ts=1/fs;        % Tiempo de muestreo

stdRuido=0.2;   % Desviación estándar de la señal discreta de ruido (gausiano)

nPuntosTexto=20;
nFFT=4*4096;    % Número de puntos de la FFT para la representación frecuencial

% Variable temporal
t=iIni:Ts:tFin;
% Portadora
c=cos(2*pi*fc*t+faseTx);
% Señal moduladora
m=cos(2*pi*fm*t);
% Señal modulada en frecuencia (FM)
sFM = fmmod(m,fc,fs,desvFreq,0);

%--------------------------------------------------------------------------
close all
clc
disp(' ')
disp('--------------------------------------------------------------- ----------------------')
disp(' Demonstrator - Frequency Modulation (FM)')
disp('--------------------------------------------------------------- ----------------------')
disp(' ')
disp('Several signals are going to be represented.')
disp('It is recommended to place the figures as they are generated')
disp('in such a way that they can all be seen simultaneously.')
disp(' ')
disp(' Press a key to start...')
pause
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Carrier signal is plotted
%--------------------------------------------------------------------------
figure(1),plot(t,c)
etiquetas(nPuntosTexto,'Time (t, seconds)','c(t)','Carrier signal')

clc
disp(' ')
disp(['The carrier signal is displayed (frequency ',num2str(fc),' Hz).'])
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.2 sec...')
pause
figure(1),xlim([2,2.2])
pause
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% The modulating signal is plotted
%--------------------------------------------------------------------------
figure(2),plot(t,m)
etiquetas(nPuntosTexto,'Time (t, sec)','m(t)','Modulating signal')
clc
disp(' ')
disp('The modulating signal, which has just been represented, is a ')
disp(['frequency sine ',num2str(fm),' Hz'])
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.2 sec...')
pause
figure(2)
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,2.2])


%--------------------------------------------------------------------------
% The PM modulating signal is plotted
%--------------------------------------------------------------------------
clc
disp(' ')
disp('The frequency modulated (FM) signal is ')
disp('the product between the carrier signal and the modulating signal.')
disp(' ')
disp(' Press a key to see it...')
pause
figure(3),plot(t,c,'r',t,sFM,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)

etiquetas(nPuntosTexto,'Time (t, sec)','s_{FM}(t)','FM modulated signal')

clc
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.2 sec...')
pause
figure(3),xlim([2,2.2])
disp(' ')
disp('In red (thin line) the carrier signal has been superimposed, to help')
disp('to understand the characteristics of this modulation')
pause

%------------------------------------------------- -------------------------
% FREQUENCY RESPONSES
%------------------------------------------------- -------------------------
clc
disp(' ')
disp('Frequency responses will be represented below.')
disp('The periodogram will be used as an estimate of densities')
disp('frequency spectra.')
disp(' ')
disp(' Press a key to see these responses...')
pause

%------------------------------------------------- -------------------------
% Modulator signal response
%------------------------------------------------- -------------------------
figure(4)
[Sm,faxis]=periodogram(m,[],nFFT,fs,'power','centered');
plot(faxis,10*log10(Sm)), grid
etiquetas(nPuntosTexto,'Frequency (Hz)','S_M(jw)','Estimation of the PSD of m(t) (periodogram)')
ylim([-60,0])
xlim([-10,10])
clc
disp(' ')
disp('This is the power spectral density of the signal')
disp('modulator (information signal)')
pause

%--------------------------------------------------------------------------
% Modulated signal
%--------------------------------------------------------------------------
figure(5)
[SsFM,faxis]=periodogram(sFM,[],nFFT,fs,'power','centered');
plot(faxis,10*log10(SsFM)), grid
etiquetas(nPuntosTexto,'Frequency (Hz)','S_S(jw)','Estimation of the PSD of s(t) (periodogram)')
clc
disp(' ')
disp('This is the power spectral density of the FM')
disp('modulated signal')
disp(' ')
disp(' Press a key to highlight the band of interest...')
pause
ylim([-60,0])
xlim([50,150])
clc
disp(' ')
disp('Note that only positive frequencies are being represented')
disp('(for negative frequencies the response is symmetrical).')
pause

